#!/bin/bash
################################################################################
#
# COPYRIGHT:
#
# (C) Copyright International Business Machines Corporation 2012. 
# Licensed Material - Property of IBM - All Rights Reserved.
#
# U.S. Government User Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
# MODIFICATION HISTORY
#
#   2012-10-16 HB Initial version
#
################################################################################
#
# DESCRIPTION
#
# Install LTFS SM fixpack. Replaced files are save to backup directory.
################################################################################


function copy_file {
    # $1: source file
    # $2: target file or target directory
    # $3: exit code in case of failure
    cp -p $1 $2 || \
    {
        echo "Failed to copy $1 to $2"
        exit $3
    }
    echo "Copied $1 to $2."
}

# get directory of this script
SCRIPT_DIR=$(dirname $0)

# get current user
SCRIPT_USER=$(id -u -n)

# make sure script is run as LTFS SM system user
if [ "$SCRIPT_USER" != "dmae" ]; then
    echo "The script needs to be run as LTFS SM system user 'dmae'."
    exit 1
fi

# make sure LTFS SM home is set
if [ -z "$ADMIRA_HOME" ]; then
    echo "Environment variable ADMIRA_HOME is not set. Please verify your environment."
    exit 2
fi

# retrieve old and new version
OLD_VERSION=$(dmae version | head -1 | cut -d' ' -f5-)
OLD_VERSION=${OLD_VERSION%.}
NEW_VERSION=$(cat ${SCRIPT_DIR}/VERSION)

# check whether we got the version information
if [ -z "$OLD_VERSION" -o -z "$NEW_VERSION" ]; then
    echo "Failed to retrieve LTFS SM version information."
    exit 3
fi

# make sure old and new version differ
if [ "$OLD_VERSION" = "$NEW_VERSION" ]; then
    echo "Fixpack is already installed."
    echo "Current LTFS Storage Manager version is: '${OLD_VERSION}'."
    exit 4
fi

# get confirmation for update
echo "================================================================================"
echo "This script will update IBM LTFS Storage Manager from"
echo "version '$OLD_VERSION' to"
echo "version '${NEW_VERSION}'."
echo "Make sure LTFS Storage Manager is down before continuing."
echo "Do you want to start the installation now [y|N]?"
    read answer
    case $answer in
        y*|Y*) ;;
        *) echo "Installation cancelled by user."
           exit 5;;
    esac

# create backup directory
echo "================================================================================"
echo
echo "Creating backup directory..."
BAK_BASEDIR=${ADMIRA_HOME}/backup
BAK_SUBDIR=$(echo ${OLD_VERSION} | tr ' ' _)--$(date +%Y-%m-%d_%H%M%S)
BAK_DIR=${BAK_BASEDIR}/${BAK_SUBDIR}

# save files to backup directory
echo
echo "Backing up files..."
for file in $(tar tfz ${SCRIPT_DIR}/fixpack.tar.gz); do
    if [ -d ${ADMIRA_HOME}/$file ]; then
        mkdir -p ${BAK_DIR}/$file || \
        {
            echo "Failed to create backup subdirectory ${BAK_DIR}/${file}."
            exit 6
        }
     elif [ -f ${ADMIRA_HOME}/${file} ]; then
         SRC_FILE=${ADMIRA_HOME}/$file
         TARGET_DIR=${BAK_DIR}/$(dirname ${file})
         copy_file $SRC_FILE $TARGET_DIR 6
     fi
done
# additionally back up DMAeServer.xml modified later
SRC_FILE=${ADMIRA_HOME}/etc/DMAeServer.xml
TARGET_DIR=${BAK_DIR}/etc
copy_file $SRC_FILE $TARGET_DIR 6

echo
echo "Compressing backup to ${BAK_BASEDIR}/${BAK_SUBDIR}.tar.gz..."
cd ${BAK_BASEDIR}/${BAK_SUBDIR}
tar cfz ${BAK_BASEDIR}/${BAK_SUBDIR}.tar.gz * || \
{
    echo "Failed to create archive ${BAK_BASEDIR}/${BAK_SUBDIR}.tar.gz."
    exit 7
}
rm -rf ${BAK_DIR}
cd - >/dev/null

echo
echo "Installing fixpack..."
tar xvzf ${SCRIPT_DIR}/fixpack.tar.gz -C${ADMIRA_HOME} || \
{
    echo "Failed to extract fixpack fixpack.tar.gz to ${ADMIRA_HOME}."
    exit 8
}

# make scripts / binaries executable
chmod a+x ${ADMIRA_HOME}/bin/maint/archive/*.sh
chmod a+x ${ADMIRA_HOME}/opt/extcmd/*.py
chmod a+x ${ADMIRA_HOME}/bin/indexer
chmod a+x ${ADMIRA_HOME}/bin/mxftool-fs
chmod a+x ${ADMIRA_HOME}/lib/java/dmae_core.jar
chmod a+x ${ADMIRA_HOME}/lib/java/ltfs_sm_admin.jar

# adapt DMAeServer.xml
echo "Adapting configuration file DMAeServer.xml..."
if [ -f ${ADMIRA_HOME}/etc/DMAeServer.xml.fixpack ]
then
  rm -f ${ADMIRA_HOME}/etc/DMAeServer.xml.fixpack
fi
awk '{print $0};/<cfg id="agent">/{print "\n            <cfg id=\"mxf_create_index\">\n                <param name=\"executable\">/home/dmae/manager/bin/indexer</param>\n            </cfg>"}' ${ADMIRA_HOME}/etc/DMAeServer.xml > ${ADMIRA_HOME}/etc/DMAeServer.xml.fixpack || \
{
    echo "Failed to amend ${ADMIRA_HOME}/etc/DMAeServer.xml"
    exit 9
}
mv ${ADMIRA_HOME}/etc/DMAeServer.xml.fixpack ${ADMIRA_HOME}/etc/DMAeServer.xml || \
{
    echo "Failed to amend ${ADMIRA_HOME}/etc/DMAeServer.xml"
    exit 10
}

# update tape type
echo "Updating tape type in database tables..."
dmae_java COM.ibm.dma.admira.cmd.tools.PostInstall || \
{
    echo "Updating database failed"
    exit 11
} 

# verify new version
INSTALLED_VERSION=$(dmae version | head -1 | cut -d' ' -f5-)
INSTALLED_VERSION=${INSTALLED_VERSION%.}
if [ "$INSTALLED_VERSION" != "$NEW_VERSION" ]; then
   echo "Updated version $INSTALLED_VERSION does not match expected version."
   echo "Expected version is: ${NEW_VERSION}".
   exit 12
fi


echo
echo "================================================================================"
echo "Installation of "
echo  "LTFS Storage Manager $NEW_VERSION succeeded."
echo "================================================================================"
exit 0

	
